package goaml.lang.state;

import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Node;

import goaml.lang.entity.Entity;

public final class SequentialState extends AbstractState {

	private final int MAX_COHORT;
	private final AbstractState[] COHORTS;
	private final boolean finiteDivision;

	protected SequentialState(final Entity entity, final Node node, final boolean canReceiveInitialValue) {
		if (node == null) {
			throw new RuntimeException("Node is null");
		}
		if (!node.hasChildNodes()) {
			throw new RuntimeException("A SequentialState must one (and only one) child !");
		}

		NamedNodeMap map = node.getAttributes();

		// Required attributes
		if (map.getNamedItem("name") != null) {
			name = map.getNamedItem("name").getTextContent();
		} else {
			throw new RuntimeException("\"name\" attribute missed");
		}

		if (map.getNamedItem("maxCohort") != null) {
			if (Integer.valueOf(map.getNamedItem("maxCohort").getTextContent()) >= 0) {
				MAX_COHORT = Integer.valueOf(map.getNamedItem("maxCohort").getTextContent());
			} else {
				throw new RuntimeException("\"maxCohort\" is negative in " + node.getNodeName() + " tag");
			}
		} else {
			throw new RuntimeException("\"maxCohort\" attribute missed");
		}
		
		if (map.getNamedItem("finiteDivision") != null) {
			finiteDivision = Boolean.valueOf(map.getNamedItem("finiteDivision").getTextContent());
		} else {
			throw new RuntimeException("\"finiteDivision\" attribute missed");
		}

		COHORTS = new AbstractState[MAX_COHORT];

		// Get the node to clone (it will be clone MAX_COHORT times)
		Node modelNode = node.getChildNodes().item(0);

		// Create every subStates
		for (int i = MAX_COHORT - 1; i >= 0; i--) {
			AbstractState cohort;
			if (i == 0 && canReceiveInitialValue == true) {
				// It's the first cohort, so if this SequentialState is itself a firstCohort of
				// another SequentialState (or there is no SequentialState above this
				// SequentialState), this cohort can receive the initial value
				cohort = AbstractState.parse(entity, modelNode, true);
			} else {
				// Not the first cohort, so it can't receive the initial value
				cohort = AbstractState.parse(entity, modelNode, false);
			}

			// set numCohort
			cohort.setNumCohort(i);

			if (i == MAX_COHORT - 1) {
					if (!finiteDivision) {
						cohort.setFollowingCohort(cohort);
					}
					else {
						cohort.setFollowingCohort(null);
					}
			} else {
				cohort.setFollowingCohort(COHORTS[i + 1]);
			}

			this.COHORTS[i] = cohort;
		}
	}

	@Override
	public double getValue() {
		double value = 0.0;

		for (int i = 0; i < MAX_COHORT; i++) {
			value += this.COHORTS[i].getValue();
		}

		return value;
	}

	public final AbstractState[] getCohorts() {
		return this.COHORTS;
	}

}
