package goaml.lang.transition;

import java.util.List;

import goaml.lang.StateChart;
import goaml.lang.parameter.Constant;
import goaml.lang.parameter.Reference;
import goaml.lang.state.State;
import goaml.lang.util.DelayedUpdate;

public final class Differentiation extends AbstractTransition {

	private Constant differentiationRate;

	public Differentiation() {
	}

	private Differentiation(final State source, final State target, final Constant differentiationRate) {
		super(source, target);
		this.differentiationRate = differentiationRate;
	}

	@Override
	protected void initialize(StateChart stateChart, State source, State target, List<Constant> constants,
			List<Reference> references) {

		// Initialize constants
		differentiationRate = Constant.getConstant("differentiationRate", constants, source, target);

		// Check if all attributes have been initialized
		if (differentiationRate == null) {
			return;
		}

		stateChart.addTransition(new Differentiation(source, target, differentiationRate));

	}

	@Override
	public void compute(StateChart stateChart) {
		// Compute the rate
		double rate = this.differentiationRate.getValue() * this.source.getValue();

		// Create DelayedUpdates in order to apply computed value on the states when all transitions
		// and processes have compute
		stateChart.addDelayedUpdate(new DelayedUpdate(this.source, DelayedUpdate.Action.SUBSTRACT, rate));
		stateChart.addDelayedUpdate(new DelayedUpdate(this.target, DelayedUpdate.Action.ADD, rate));
	}

}
