package goaml.lang.transition;

import java.util.List;

import goaml.lang.StateChart;
import goaml.lang.parameter.Constant;
import goaml.lang.parameter.Reference;
import goaml.lang.state.State;
import goaml.lang.util.DelayedUpdate;

public class Migration extends AbstractTransition
{
	private static final long serialVersionUID = 1L;

	private static double MAGIC = 100.0;
	
	private Constant MigrationRate1;
	private Constant MigrationRate2;
	private Constant MigrationRate3;

	public Migration() {}

	private Migration(final State source, final State target, final Constant MigrationRate1, final Constant MigrationRate2, 
				final Constant MigrationRate3) 
	{
		super(source, target);
		this.MigrationRate1 = MigrationRate1;
		this.MigrationRate2 = MigrationRate2;
		this.MigrationRate3 = MigrationRate3;
	}
	

	@Override
	protected void initialize(StateChart stateChart, State source, State target, List<Constant> constants,
			List<Reference> references) {

		// Initialize constants
		MigrationRate1 = Constant.getConstant("MigrationRate1", constants, source, target);
		MigrationRate2 = Constant.getConstant("MigrationRate2", constants, source, target);
		MigrationRate3 = Constant.getConstant("MigrationRate3", constants, source, target);

		// Check if all attributes have been initialized
		if ((MigrationRate1 == null) || (MigrationRate2 == null) || (MigrationRate3 == null)) {
			return;
		}

		stateChart.addTransition(new Migration(source, target, MigrationRate1, MigrationRate2, MigrationRate3));

	}

	@Override
	public void compute(StateChart stateChart) {
		// Compute the rate
		double constant = Math.pow(MigrationRate3.getValue() * MigrationRate2.getValue(), MigrationRate1.getValue());
		double rate = Math.min(MAGIC, constant) * this.source.getValue();

		// Create DelayedUpdates in order to apply computed value on the states when all transitions
		// and processes have compute
		stateChart.addDelayedUpdate(new DelayedUpdate(this.source, DelayedUpdate.Action.SUBSTRACT, rate));
		stateChart.addDelayedUpdate(new DelayedUpdate(this.target, DelayedUpdate.Action.ADD, rate));
	}

}
