package goaml.lang.transition;

import java.util.List;

import goaml.lang.StateChart;
import goaml.lang.parameter.Constant;
import goaml.lang.parameter.Reference;
import goaml.lang.state.State;
import goaml.lang.util.DelayedUpdate;

public final class Suppression extends AbstractTransition {

	private Constant suppressionRate;
	private Reference treg;

	public Suppression() {
	}

	private Suppression(final State source, final State target, final Constant suppressionRate, final Reference treg) {
		super(source, target);
		this.suppressionRate = suppressionRate;
		this.treg = treg;
	}

	@Override
	protected void initialize(StateChart stateChart, State source, State target, List<Constant> constants,
			List<Reference> references) {

		// Initialize constants
		suppressionRate = Constant.getConstant("suppressionRate", constants, source, target);

		for (Reference reference : references) {
			if (reference.getName().equals("treg")) {
				treg = reference;
			}

			// Check if all attributes have been initialized
			if (suppressionRate == null || treg == null) {
				return;
			}

			stateChart.addTransition(new Suppression(source, target, suppressionRate, treg));
		}
	}

	@Override
	public void compute(StateChart stateChart) {
		// Compute the rate
		double rate = this.suppressionRate.getValue() * this.source.getValue() * this.treg.getState().getValue();

		// Create DelayedUpdates in order to apply computed value on the states when all transitions
		// and processes have compute
		stateChart.addDelayedUpdate(new DelayedUpdate(this.source, DelayedUpdate.Action.SUBSTRACT, rate));
		stateChart.addDelayedUpdate(new DelayedUpdate(this.target, DelayedUpdate.Action.ADD, rate));
	}

}
